using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Text.Unicode;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;
using Microsoft.Win32.SafeHandles;

namespace FitConnect.Encryption;

public class CertificateHelper {
    private readonly ILogger? _logger;

    public CertificateHelper(ILogger? logger = null) {
        _logger = logger;
    }

    internal bool ValidateCertificate(string keyJson, LogLevel logLevel) =>
        ValidateCertificate(new JsonWebKey(keyJson), logLevel);

    internal bool ValidateCertificate(JsonWebKey key, LogLevel logLevel = LogLevel.Error) {
        var certificates = key.X5c.Select(s => new X509Certificate2(Convert.FromBase64String(s)))
            .ToList();

        X509Certificate2? root = null; //new X509Certificate2("./certificates/root.pem");


        _logger?.LogTrace("Found {Count} certificate(s)", certificates.Count);

        var valid = certificates.Aggregate(true,
            (result, cert) => result
                              && ValidateCertificate(cert, out _,
                                  root,
                                  logLevel)
                              && cert.Verify()
        );
        return valid;
    }

    internal bool ValidateCertificate(X509Certificate2 certificate,
        out X509ChainStatus[] chainStatus,
        X509Certificate2? rootCertificate = null,
        LogLevel logLevel = LogLevel.Warning) {
        var certificateChain = new X509Chain();


        if (rootCertificate != null) {
            certificateChain.ChainPolicy.CustomTrustStore.Add(rootCertificate);
            certificateChain.ChainPolicy.TrustMode = X509ChainTrustMode.CustomRootTrust;
        }

        certificateChain.ChainPolicy.RevocationMode = X509RevocationMode.Online;
        certificateChain.ChainPolicy.RevocationFlag = X509RevocationFlag.ExcludeRoot;
        certificateChain.ChainPolicy.VerificationFlags = X509VerificationFlags.NoFlag;
        certificateChain.ChainPolicy.UrlRetrievalTimeout = new TimeSpan(0, 0, 30);

        var result = certificateChain.Build(certificate);

        chainStatus = certificateChain.ChainStatus;

        var statusAggregation = certificateChain.ChainStatus.Aggregate("",
            (r, s) => r + "\n\t - " + s.Status + ": " + s.StatusInformation);

        if (!string.IsNullOrWhiteSpace(statusAggregation))
            _logger?.Log(logLevel, "Certificate status: {ObjStatusInformation}",
                statusAggregation);
        return result;
    }
}
